<?
# PHP Weathermap 0.4 - Pre-release software. For testing
# Copyright Howard Jones, 2005 howie@thingy.com
# http://wotsit.thingy.com/haj/cacti/
# Released under the GNU Public License

    require_once "HTML_ImageMap.class.php";
    
    # Big chunks of the code in here are for the (forthcoming) editor
    # - the WeatherMap and it's Nodes and Links know how to write out a config as well 
    #   as read one, and also how to do server-side hit-testing.
    # - it should be easy enough to add new methods to load and save config information
    #   to a database too, for tighter integration with cacti in the future.
    
    # some of it is still very much a literal translation of the original perl. Sorry about that.
    # this is my first reasonable-sized php project from scratch
    
    class WeatherMapNode {
        var $x, $y;
        var $label;
        var $name;
        var $infourl;
        var $overliburl;
        var $maphtml;
        var $selected=0;
        var $iconfile;
                
        function WriteConfig($fd)
        {
            fwrite($fd,"NODE ".$this->name."\n");
            fwrite($fd,"\tPOSITION ".$this->x." ".$this->y."\n");
            fwrite($fd,"\tLABEL ".$this->label."\n");
            if($this->infourl != "")
            {
                fwrite($fd,"\tINFOURL ".$this->infourl."\n");
            }
            if($this->overliburl != "")
            {
                fwrite($fd,"\OVERLIBGRAPH ".$this->overliburl."\n");
            }
            if($this->iconfile != "")
            {
                fwrite($fd,"\tICON ".$this->iconfile."\n");
            }
            fwrite($fd,"\n");
        }
        
        function Draw($im, &$map)
        {        
            if($this->iconfile != "")
            {
                // draw the supplied icon, instead of the labelled box
                $temp_im = imagecreatefrompng($this->iconfile);
                if($temp_im)
                {
                    $w = imagesx($temp_im);
                    $h = imagesy($temp_im);
                    $x1 = $this->x - $w/2;
                    $y1 = $this->y - $h/2;
                    $x2 = $this->x + $w/2;
                    $y2 = $this->y + $h/2;
                  
                    imagecopymerge($im,$temp_im,$x1,$y1,0,0,$w,$h,100);
                    $map->imap->addArea("Rectangle","NODE:".$this->name,"",array($x1,$y1,$x2,$y2));
                }
                else
                {
                    $map->warn("Couldn't open PNG ICON: ".$this->iconfile."\n");
                }
            }
            elseif($this->label != "")
            {
                $padding = 0;
                $font=4;
     
                $strwidth = imagefontwidth($font) * strlen($this->label);
                $strheight = imagefontheight($font);
                
                $x = $this->x;
                $y = $this->y;
                
                $x1 = $x - ($strwidth/2)-2;
                $x2 = $x + ($strwidth/2) +2;
                $y1 = $y - ($strheight/2) -2;
                $y2 = $y + ($strheight/2) +2;
                
                imagefilledrectangle($im, $x1, $y1, $x2, $y2, $map->white);
        
                if($this->selected)
                {
                    imagerectangle($im, $x1, $y1, $x2, $y2, $map->selected);
                    // would be nice if it was thicker, too...
                    imagerectangle($im, $x1-1, $y1-1, $x2+1,$y2+1, $map->selected);
                }
                else
                {
                    imagerectangle($im, $x1, $y1, $x2, $y2, $map->black);
                }
                imagestring($im,$font,$x-$strwidth/2, $y-$strheight/2+1,$this->label,$map->black);
                
                $map->imap->addArea("Rectangle","NODE:".$this->name,"",array($x1,$y1,$x2,$y2));
            }
        }
    };
    
    class WeatherMapLink {
        var $name;
        var $maphtml;
        var $a,$b;  // the ends - references to nodes
        var $width = 7;
        var $overliburl = "";
        var $infourl = "";
        var $bandwidth_in = 0;
        var $bandwidth_out = 0;
        var $max_bandwidth_in = 10240;
        var $max_bandwidth_out = 10240;
        var $target = "";
        var $selected=0;
        var $inpercent, $outpercent;
                
        function Draw($im, &$map)
        {
    		$max_bw_in = $this->max_bandwidth_in;
	       	$max_bw_out = $this->max_bandwidth_out;

            $pc = intval((($this->bandwidth_out)/($max_bw_out))*100);
            if($pc>100) {$pc = 100;} // should't be needed, but still...
            $this->outpercent = $pc; 
              
            $pc = intval((($this->bandwidth_in)/($max_bw_in))*100);
            if($pc>100) {$pc = 100;} // this happens sometimes, if you mis-specify the bw
            $this->inpercent = $pc; 
            
            $x1 = $this->a->x;
            $y1 = $this->a->y;

            $x2 = $this->b->x;
            $y2 = $this->b->y;

            $midx = $map->Middle($x1,$x2);
            $midy = $map->Middle($y1,$y2);
            
            $outline_colour = $map->black;
            if($this->selected)
            {
                $outline_colour = $map->selected;
            }
            
            # my attempt at removing duplicated code
            
            
            $inbound = array($x1,$y1,$midx,$midy,$this->outpercent,$this->bandwidth_out);
            $outbound = array($x2,$y2,$midx,$midy,$this->inpercent,$this->bandwidth_in);

            if($map->sizedebug)
            {
                $inbound = array($x1,$y1,$midx,$midy,$this->outpercent,$this->max_bandwidth_out);
                $outbound = array($x2,$y2,$midx,$midy,$this->inpercent,$this->max_bandwidth_in);
            }
                    
            foreach ( array($inbound,$outbound) as $task)
            {
                $labelx = $map->Middle($task[0],$task[2]);
                $labely = $map->Middle($task[1],$task[3]);
                
                $link_colour = $map->ColourFromPercent($task[4]);
                if($map->widthmod)
                {
                    // for bulging animations
                    $link_width = ($this->width*$task[4]*1.5)/100;
                }
                else
                {
                    $link_width = $this->width;
                }
                
                $this->DrawArrow($im, $task[0],$task[1],$task[2],$task[3],$link_width,1,$link_colour,$map);
                $this->DrawArrow($im, $task[0],$task[1],$task[2],$task[3],$link_width,0,$outline_colour,$map);
                
                if($map->labelstyle == 'bits')
                {
                    $thelabel = $map->nice_bandwidth($task[5]);
                }
                else
                {
                    $thelabel = $task[4]."%";
                }
                $map->DrawLabel($im, $labelx, $labely, $thelabel, 2, 0, "");
              //  $map->imap->addArea("Polygon",$this->name,"",array($task[0],$task[1],$task[2],$task[3]) );

            }
        }
            
       	function RotX($a,$b,$x,$y)
        {
            return  cos( atan2($y,$x) + atan2($b,$a) ) * sqrt( $x*$x + $y*$y );
        }
    
        function RotY($a,$b,$x,$y)
        {
            return sin( atan2($y,$x) + atan2($b,$a) ) * sqrt( $x*$x + $y*$y ) ;
        }

                
        function DrawArrow($im, $x1,$y1,$x2,$y2,$w,$solid,$colour,&$map)
        {
        
            $poly = array();
        
            $poly[] = $x1 + $this->RotX($x2-$x1, $y2-$y1, 0, $w);
            $poly[] = $y1 + $this->RotY($x2-$x1, $y2-$y1, 0, $w);
    
            $poly[] = $x2 + $this->RotX($x2-$x1, $y2-$y1, -4*$w, $w);
            $poly[] = $y2 + $this->RotY($x2-$x1, $y2-$y1, -4*$w, $w);
    
            $poly[] = $x2 + $this->RotX($x2-$x1, $y2-$y1, -4*$w, 2*$w);
            $poly[] = $y2 + $this->RotY($x2-$x1, $y2-$y1, -4*$w, 2*$w);
    
            $poly[] = $x2;
            $poly[] = $y2;
    
            $poly[] = $x2 + $this->RotX($x2-$x1, $y2-$y1, -4*$w, -2*$w);
            $poly[] = $y2 + $this->RotY($x2-$x1, $y2-$y1, -4*$w, -2*$w);
    
            $poly[] = $x2 + $this->RotX($x2-$x1, $y2-$y1, -4*$w, -$w);
            $poly[] = $y2 + $this->RotY($x2-$x1, $y2-$y1, -4*$w, -$w);
    
            $poly[] = $x1 + $this->RotX($x2-$x1, $y2-$y1, 0, -$w);
            $poly[] = $y1 + $this->RotY($x2-$x1, $y2-$y1, 0, -$w);
    
            if($solid)
            {
                imagefilledpolygon($im,$poly,7,$colour);
            }
            else
            {
                imagepolygon($im,$poly,7,$colour);
            }

            // Really, I'd rather this was done some other way, but this is so each area gets done once only
            if($solid==1)
            {
                $map->imap->addArea("Polygon","LINK:".$this->name,"",$poly );
            }         
        }

                
        function WriteConfig($fd)
        {
            fwrite($fd,"LINK ".$this->name."\n");
            fwrite($fd,"\tNODES ".$this->a->name." ".$this->b->name."\n");

            if($this->target != "")
            {
                fwrite($fd,"\tTARGET ".$this->target."\n");
            }

            if($this->infourl != "")
            {
                fwrite($fd,"\tINFOURL ".$this->infourl."\n");
            }
            
            if($this->overliburl != "")
            {
                fwrite($fd,"\tOVERLIBGRAPH ".$this->overliburl."\n");
            }
            
            if($this->maxbandwidth_in == $this->max_bandwidth_out)
            {
                fwrite($fd,"\tBANDWIDTH ".$this->max_bandwidth_in."\n");
            }
            else
            {
                fwrite($fd,"\tBANDWIDTH ".$this->max_bandwidth_in." ".$this->max_bandwidth_out."\n");
            }
            if($this->width != 7)
            {
                fwrite($fd,"\tWIDTH ".$this->width."\n");
            }
            fwrite($fd,"\n");
        }
    };

    class WeatherMap {
        var $nodes = array();   // an array of WeatherMapNodes
        var $links = array();   // an array of WeatherMapLinks
        var $labelstyle = "percent";
        var $background = "";
        var $width = 800;
        var $height = 600;
        var $keyx = -1;
        var $keyy = -1;
        var $htmlstyle = 'static';
        var $imap;
        var $colours;
        var $configfile;
        var $imagefile;
        var $imageuri;
        var $rrdtool;
        var $debugging;
        var $title;
        var $kilo = 1000;
        var $sizedebug = FALSE;
        var $widthmod = FALSE;
                
        var $black,$white,$grey,$selected;
        
        function WeatherMap()
        {
            $this->Reset();
        }
        
        function Reset()
        {
            $this->debugging=FALSE;
            $this->nodes = array();   // an array of WeatherMapNodes
            $this->links = array();   // an array of WeatherMapLinks
            $this->labelstyle = "percent";
            $this->background = "";
            $this->width = 800;
            $this->height = 600;
            $this->keyx = -1;
            $this->keyy = -1;
            $this->htmlstyle = 'static';
            $this->imap = new HTML_ImageMap('weathermap');
            $this->colours=array();
            $this->configfile="";
            $this->imagefile="";
            $this->imageuri="";
            $this->title="Network Weathermap";
            $this->kilo=1000;
        }
        
        function debug($string)
        {
            if($this->debugging)
            {
                $stderr = fopen('php://stderr', 'w'); 
                fwrite($stderr,$string);
                fclose($stderr);
            }
        }
        
        function warn($string)
        {
            $stderr = fopen('php://stderr', 'w'); 
            fwrite($stderr,$string);
            fclose($stderr);
        }
        
        function RandomData()
        {
            foreach ($this->links as $link)
            {
                $this->links[$link->name]->bandwidth_in = rand(0,$link->max_bandwidth_in);
                $this->links[$link->name]->bandwidth_out = rand(0,$link->max_bandwidth_out);
            }
        }
        function ReadFromMRTG($target)
        {                
            $this->debug("Opening $target \n");

            $inbw = -1;
            $outbw = -1;
           $fd = fopen($target,"r");
            if($fd)
            {
                while (!feof($fd))
                {
                    $buffer = fgets($fd,4096);
                    
                    if(preg_match("/<\!-- cuin d (\d+) -->/",$buffer,$matches) )
                    {
                        $inbw = $matches[1]*8;
                    }
                    
                    if(preg_match("/<\!-- cuout d (\d+) -->/",$buffer,$matches) )
                    {
                        $outbw = $matches[1]*8;
                    }
                }
            }
            else
            {
                // some error code to go in here
                $this->debug("Couldn't open ($target). \n");
            }
            $this->debug("ReadFromMRTG: Returning ($inbw,$outbw)\n");
            return( array($inbw,$outbw) );
        }
        
        function ReadFromRRD($target,$in_ds="traffic_in",$out_ds="traffic_out")
        {
            $inbw = -1;
            $outbw = -1;
            
            // rrdtool fetch RRDfile.rrd AVERAGE --start now

            // we get the last 400 seconds of data - this might be 1 or 2 lines, depending on when in the 
            // cacti polling cycle we get run. This ought to stop the 'some lines are grey' problem that some
            // people were seeing

		$command = $this->rrdtool." fetch '$target' AVERAGE --start now-400";
		$this->debug("ReadFromRRD: Running: $command\n");
            $pipe = popen($command,"r");
            $lines = array();
		$count = 0;
            if(isset($pipe))
            {              
                $headings = fgets($pipe,4096);
                fgets($pipe,4096); // skip the blank line
                $buffer="";
                while(!feof($pipe))
                {
                    $line = fgets($pipe,4096);
                    $this->debug("> ".$line);
                    $buffer .= $line;
                    $lines[] = $line;
                };  
                
                $rlines = array_reverse($lines);
		        $gotline = 0;
                $theline = "";
                
                foreach ($rlines as $line)
                {
                    $this->debug("--".$line."\n");
                    $cols = preg_split("/\s+/",$line);
                    $dataok = 1;
                    foreach ($cols as $col)
                    {
                        if($col=='nan') { $dataok=0; }
                    }
                    if($gotline==0 && $dataok==1 && trim($line) != "")
                    {
                        $this->debug("ReadFromRRD: Found a good line: $line ($headings)\n");
			             $theline = $line;
			             $gotline = 1;
			             $countwas = $count;
                    }
			         $count++;
                }                           
                    
                pclose($pipe);
            }
		if($theline != "")
		{
			if($countwas>2)
			{
				$this->warn("ReadFromRRD: Data is not most recent entry ($countwas) for link: $target\n");
			}
            		$this->debug("ReadFromRRD: Our line is $theline\n");
                    	$cols = preg_split("/\s+/",$line);
			$heads = preg_split("/\s+/",$headings); 
			# $values = array_combine($heads,$cols);
			for ($i = 0, $cnt = count($cols); $i < $cnt; $i++) {
			            $values[$heads[$i]] = $cols[$i];
        }

			$inbw = $values[$in_ds]*8;		
			$outbw = $values[$out_ds]*8;		
		}
            $this->debug("ReadFromRRD: Returning ($inbw,$outbw)\n");

            return ( array($inbw, $outbw) );
        }

        function ReadData()
        {
            $this->debug("\n=========\nReadData: Updating link data for all links\n");     

            foreach ($this->links as $link)
            {
                $name = $link->name;
                $target = $link->target;
                if($target != "")
                {
                    $this->debug("ReadData: Updating link $name from $target\n");     
                    if(preg_match("/\.rrd/",$target))
                    {   // Direct from RRD
                        list($in,$out) = $this->ReadFromRRD($target);
                    }
                    else
                    {   // HTML/MRTG style 
                        list($in,$out) = $this->ReadFromMRTG($target);
                    }
                    
                    if( ($in < 0) || ($out < 0) )
                    {
                        $in = 0; 
                        $out = 0;
                        $this->warn("ReadData: Link $name had no valid data\n");                   
                    }

                    $this->links[$name]->bandwidth_in = $in;
                    $this->links[$name]->bandwidth_out = $out;
                    $this->debug("ReadDate: Setting $in,$out\n");
                }        
                else
                {
                    $this->warn("ReadData: Link $name has no TARGET defined\n");
                }
                $this->debug("--------------\n");
            }
        }
        
        
        function Distance($x1,$x2)
        {
            return( sqrt( ($x1*$x1) + ($x2*$x2) ) );
        }  
                	
        function Middle($x1,$x2)
        {
           return( $x1 + ($x2-$x1)/2);
        }
                
        function DrawLabel($im, $x,$y,$text,$font,$padding,$nodename)
        {            
            $strwidth = imagefontwidth($font) * strlen($text);
            $strheight = imagefontheight($font);
            
            $x1 = $x - ($strwidth/2) - $padding -2;
            $x2 = $x + ($strwidth/2) + $padding +2;
            $y1 = $y - ($strheight/2) - $padding -2;
            $y2 = $y + ($strheight/2) + $padding +2;
            
            imagefilledrectangle($im, $x1, $y1, $x2, $y2, $this->white);
            imagerectangle($im, $x1, $y1, $x2, $y2, $this->black);
    
            imagestring($im,$font,$x-$strwidth/2, $y-$strheight/2+1,$text,$this->black);
            
            # Generate some HTML, if that's what is required
            if($nodename != "")
            { 
                $html = '<AREA NAME="'.$nodename.'" SHAPE="RECT" COORDS="';
                $html .= "$x1,$y1,$x2,$y2";
                $html .= '">';
                
                $this->nodes[$nodename]->maphtml .= $html;
                
                # $html .= extra_map_html($nodes[$nodename],$nodename);
                # $html .= '>';
    
                # XXX - todo
               # $nodes[$nodename]['HTML'] .= $html;
            }

        }

                
        function ColourFromPercent($percent)
        {
            $colours = $this->colours;
            if($percent>100) {$percent = 100; }
            
            foreach ($colours as $key => $colour)
            {              
                if( ($percent >= $colour['bottom']) and ($percent <= $colour['top']) )
                {
                    return $colour['gdref'];
                } 
            }
            
            // you'll only get grey for a COMPLETELY quiet link
            if($percent==0) {return $this->grey; }
           
            // and you'll only get white for a link with no colour assigned
            return $this->white;
        }
        
        function DrawLink($im, $outlinecolour,$width,$link,$linkname)
        {
        
            $x1 = $link->a->x;
            $y1 = $link->a->y;

            $x2 = $link->b->x;
            $y2 = $link->b->y;

        
            $midx = $this->Middle($x1,$x2);
            $midy = $this->Middle($y1,$y2);
        
            $lab1x = $this->Middle($x1,$midx);
            $lab1y = $this->Middle($y1,$midy);
            $lab2x = $this->Middle($x2,$midx);
            $lab2y = $this->Middle($y2,$midy);
            
            $link_colour = $this->ColourFromPercent($link['outpercent']);
            
            $this->DrawArrow($im, $x1,$y1,$midx,$midy,$width,1,$link_colour,"");
            $this->DrawArrow($im, $x1,$y1,$midx,$midy,$width,0,$outlinecolour,$linkname);
            if($this->labelstyle == 'bytes')
            {
                $this->DrawLabel($im, $lab1x, $lab1y, nice_bandwidth($link->bandwidth_out),2,0,"");
            }
            else
            {
                $this->DrawLabel($im, $lab1x, $lab1y, $link['outpercent']."%",2,0,"");
            }
            
            $link_colour = $this->ColourFromPercent($link['inpercent']);
    
            $this->DrawArrow($im, $x2,$y2,$midx,$midy,$width,1,$link_colour,"");
            $this->DrawArrow($im, $x2,$y2,$midx,$midy,$width,0,$outlinecolour,$linkname);
            if($this->labelstyle == 'bytes')
            {
                $this->DrawLabel($im, $lab2x, $lab2y, nice_bandwidth($link['inbw']),2,0,"");
            }
            else
            {
               $this->DrawLabel($im, $lab2x, $lab2y, $link['inpercent']."%",2,0,"");
            }
        }
        
        function coloursort($a,$b)
        {
            if($a['bottom'] == $b['bottom']) { return 0;}
            
            if($a['bottom'] < $b['bottom']) { return -1;}
            
            return 1;
        }
        
        function DrawLegend($im)
        {
            $title = "Traffic Load";
     
            $colours = $this->colours;
            $nscales = count($colours);
            
            $font = 4;
         
            $x = $this->keyx;
            $y = $this->keyy;
            
            if( ($x >=0) && ($y >= 0) )
            {
                $boxwidth = imagefontwidth($font) * strlen($title) + 10; 
                $boxheight = imagefontheight($font) * ($nscales+1) + 10;
        
                $boxx = $x;
                $boxy = $y;
                # allow for X11-style negative positioning
                if($boxx < 0) { $boxx += $this->width; }
                if($boxy < 0) { $boxy += $this->height; }
        
                imagefilledrectangle($im, $boxx, $boxy, $boxx+$boxwidth, $boxy+$boxheight, $this->white);
                imagerectangle($im, $boxx, $boxy, $boxx+$boxwidth, $boxy+$boxheight, $this->black);
                imagestring($im, $font, $boxx+4, $boxy+4,$title, $this->black);
        
                usort ($colours,array("Weathermap","coloursort"));
        
                $i = 1;
                foreach ($colours as $colour)
                {
        
                    imagefilledrectangle($im,
                                $boxx + 6, $boxy + imagefontheight($font)*$i + 8,
                                $boxx + 6 + 16, $boxy + imagefontheight($font)*($i+1) + 6,
                                $colour['gdref']
                                );
                                
                    $labelstring = sprintf("%s-%s%%",$colour['bottom'],$colour['top']);
                    imagestring($im, $font, $boxx+6+20, $boxy+8+imagefontheight($font)*$i,$labelstring,$this->black);
                    $i++;
                }
            }
        }
            
        function DrawTimestamp($im,$font,$colour)
        {            
            # add a timestamp to the corner, so we can tell if it's all being updated
           $datestring = "Created: ".date("M d Y H:i:s",time());
           imagestring($im,$font,
                       $this->width - (strlen($datestring)) * imagefontwidth($font),0,
                       $datestring, $colour);
        }
        
        function ReadConfig($filename)
        {
            $last_seen = "---";
            $fd = fopen($filename,"r");
            if($fd)
            {
                $linecount=0;
                while (!feof($fd))
                {
                
                    $buffer = fgets($fd,4096);
                    $linematched=0;
                    $linecount++;
                    
                    if(preg_match("/^\s*#/",$buffer) )
                    {
                        // this is a comment line
                    }
                    else
                    {
                    
                        if(preg_match("/^\s*LINK\s+(\S+)/i",$buffer, $matches))
                        {
                            if($last_seen == "NODE") { $this->nodes[$curnode->name] = $curnode; }
                            if($last_seen == "LINK") { $this->links[$curlink->name] = $curlink; }
    
                            $curlink = new WeatherMapLink;
                            $curlink->name = $matches[1];
        
                            $last_seen = "LINK";
                            $linematched++;
                        }
    
                        
                        if(preg_match("/^\s*NODE\s+(\w+)/i",$buffer, $matches))
                        {
                            if($last_seen == "NODE") { $this->nodes[$curnode->name] = $curnode; }
                            if($last_seen == "LINK") { $this->links[$curlink->name] = $curlink; }
                            $last_seen = "NODE";
                            $curnode = new WeatherMapNode;
                            $curnode->name = $matches[1];
                            $linematched++;
                        }
                        
                        if(preg_match("/^\s*POSITION\s+(\d+)\s+(\d+)/i",$buffer, $matches))
                        {
                            if($last_seen=='NODE')
                            {
                                $curnode->x = $matches[1];
                                $curnode->y = $matches[2];
                                $linematched++;
                            }
                        }
                    
                        if(preg_match("/^\s*LABEL\s+(.*)/i",$buffer,$matches))
                        {
                            if($last_seen=='NODE')
                            {
                                $curnode->label = $matches[1];
                                $linematched++;
                            }
                        }
    
                        if(preg_match("/^\s*NODES\s+(\S+)\s+(\S+)/i",$buffer, $matches))
                        {
                            if($last_seen=='LINK')
                            {
                                $curlink->a = $this->nodes[$matches[1]];
                                $curlink->b = $this->nodes[$matches[2]];
                                $linematched++;
                            }
                        }
                        
                        if(preg_match("/^\s*TARGET\s+(\S+)/i",$buffer,$matches))
                        {
                            if($last_seen=='LINK')
                            {
                                $curlink->target = $matches[1];
                                $linematched++;
                            }
                        }
                        
                        if(preg_match("/^\s*WIDTH\s+(\d+)/i",$buffer,$matches))
                        {
                            if($last_seen=='LINK')
                            {
                                $curlink->width = $matches[1];
                                $linematched++;
                            }
		              		else
	               			{
                                $this->width = $matches[1];
                                $linematched++;
	               			}
                        }
                                                
                        if(preg_match("/^\s*HEIGHT\s+(\d+)/i",$buffer,$matches))
                        {
				            $this->height = $matches[1];
				            $linematched++;
			             }
	
                        if(preg_match("/^\s*BANDWIDTH\s+(\d+[KMGT]*)\s*$/i",$buffer,$matches))
                        {
                            if($last_seen=='LINK')
                            {
                                $curlink->max_bandwidth_in = $this->unformat_number($matches[1]);
                                $curlink->max_bandwidth_out =  $this->unformat_number($matches[1]);
                                $linematched++;
                            }
                        }
                        
                        if(preg_match("/^\s*BANDWIDTH\s+(\d+[KMGT]*)\s+(\d+[KMGT]*)$/i",$buffer,$matches))
                        {
                            if($last_seen=='LINK')
                            {
                                $curlink->max_bandwidth_in = $this->unformat_number($matches[1]);
                                $curlink->max_bandwidth_out =  $this->unformat_number($matches[2]);
                                $linematched++;
                            }
                        }
                        
                        if(preg_match("/^\s*ICON\s+(\S+)\s*$/i",$buffer,$matches))
                        {
                            if($last_seen=='NODE')
                            {
                                $curnode->iconfile = $matches[1];
                                $linematched++;
                            }
                        }                 
                        
                        if(preg_match("/^\s*INFOURL\s+(\S+)/i",$buffer,$matches))
                        {
                            if($last_seen == "LINK")
                            {
                                $curlink->infourl = $matches[1];
                            }
                            else
                            {
                                $curnode->infourl = $matches[1];
                            }
                            $linematched++;
                        }
    
                        if(preg_match("/^\s*OVERLIBGRAPH\s+(\S+)/i",$buffer,$matches))
                        {
                            if($last_seen == "LINK")
                            {
                                $curlink->overliburl = $matches[1];
                            }
                            else
                            {
                                $curnode->overliburl = $matches[1];
                            }
                            $linematched++;
                        }
    
                        if(preg_match("/^\s*SCALE\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/i",$buffer,$matches))
                        {
                            $key = $matches[1].':'.$matches[2];
                            $this->colours[$key]['bottom'] = $matches[1];
                            $this->colours[$key]['top'] = $matches[2];
                            $this->colours[$key]['red'] = $matches[3];
                            $this->colours[$key]['green'] = $matches[4];
                            $this->colours[$key]['blue'] = $matches[5];
                            $this->colours[$key]['key'] = $key;
                            $linematched++;
                        }
                     
                        if(preg_match("/^\s*KEYPOS\s+(\d+)\s+(\d+)/i",$buffer,$matches))
                        {
                            $this->keyx = $matches[1];
                            $this->keyy = $matches[2];
                            $linematched++;
                        }
                        
                        if(preg_match("/^\s*BWLABELS\s+(bits|percent)/i",$buffer, $matches))
                        {
                            $this->labelstyle = $matches[1];
                            $linematched++;
                        }

                        if(preg_match("/^\s*KILO\s+(\d+)/i",$buffer, $matches))
                        {
                            $this->kilo = $matches[1];
                            $linematched++;
                        }
    
                        if(preg_match("/^\s*BACKGROUND\s+(\S+)/i",$buffer, $matches))
                        {
                            $this->background = $matches[1];
                            $linematched++;
                        }
                        
                        if(preg_match("/^\s*HTMLSTYLE\s+(static|overlib)/i",$buffer, $matches))
                        {
                            $this->htmlstyle = $matches[1];
                            $linematched++;
                        }
                        
                        if(preg_match("/^\s*TITLE\s+(.*)/i",$buffer, $matches))
                        {
                            $this->title = $matches[1];
                            $linematched++;
                        }
                        
                        if($linematched==0 && trim($buffer) != "")
                        {   
                                $this->warn("Unrecognised config on line $linecount: $buffer");
                        }
                    } // if blankline
                } // while

                if($last_seen == "NODE") { $this->nodes[$curnode->name] = $curnode; }
                if($last_seen == "LINK") { $this->links[$curlink->name] = $curlink; }
                             
            } // if $fd
            else
            {
                die("Couldn't open config file $filename for reading"); 
            }
            
            fclose($fd);
        
            if(count($this->colours)==0)
            {
                // load some default colouring, otherwise it all goes wrong
                $defaults = array(
                                '1:10' => array('bottom'=>1,'top'=>10,'red'=>140,'green'=>0,'blue'=>255),
                                '10:25' => array('bottom'=>10,'top'=>25,'red'=>32,'green'=>32,'blue'=>255),
                                '25:40' => array('bottom'=>25,'top'=>40,'red'=>0,'green'=>192,'blue'=>255),
                                '40:55' => array('bottom'=>40,'top'=>55,'red'=>0,'green'=>240,'blue'=>0),
                                '55:70' => array('bottom'=>55,'top'=>70,'red'=>240,'green'=>240,'blue'=>0),
                                '70:85' => array('bottom'=>70,'top'=>85,'red'=>255,'green'=>192,'blue'=>0),
                                '85:100' => array('bottom'=>85,'top'=>100,'red'=>255,'green'=>0,'blue'=>0)
                                ) ;
                foreach ($defaults as $key => $def)
                {
                    $this->colours[$key] = $def;
                }
            }
            $this->configfile = "$filename";
        }
        
        function WriteConfig($filename)
        {

            $fd = fopen($filename,"w");
            if($fd)
            {
                fwrite($fd,"# Automatically generated by php-weathermap\n\n");
            
                if($this->background != "")
                {
                    fwrite($fd,"BACKGROUND ".$this->background."\n");               
                }
                else
                {
                    fwrite($fd,"WIDTH ".$this->width."\n");
                    fwrite($fd,"HEIGHT ".$this->height."\n");
                }
                fwrite($fd,"HTMLSTYLE ".$this->htmlstyle."\n");
                fwrite($fd,"BWLABELS ".$this->labelstyle."\n");
                fwrite($fd,"KILO ".$this->kilo."\n");

                if(trim($this->title != ""))
                {
                    fwrite($fd,"TITLE ".$this->title."\n");
                }
                
                if( ($this->keyx >= 0) && ($this->keyy >= 0) )
                {
                    fwrite($fd,"KEYPOS ".$this->keyx." ".$this->keyy."\n");
                }
                
                $colours = $this->colours;

                fwrite($fd,"\n#     low  high   red green blue\n");
                foreach ($colours as $colour)
                {
                    fwrite($fd, sprintf("SCALE %d %d %d %d %d\n", $colour['bottom'], $colour['top'], $colour['red'],$colour['green'],$colour['blue']) );
                }
                
                fwrite($fd,"\n# End of global section\n\n# Node definitions:\n");
                
                foreach ($this->nodes as $node)
                {
                    $node->WriteConfig($fd);
                }
                fwrite($fd,"\n# End of NODE section\n\n# Link definitions:\n");

                foreach ($this->links as $link)
                {
                    $link->WriteConfig($fd);
                }
                fwrite($fd,"\n# End of LINK section\n\n# That's All Folks!\n");          
            }
            else
            {
                print "Couldn't open config file $filename for writing";
                die(); 
            }
        }
        
        function AllocateColours($im)
        {        
            $colours = $this->colours;
            
            foreach ($colours as $key => $colour)
            {
              $this->colours[$key]['gdref'] = imagecolorallocate($im,$colour['red'],$colour['green'],$colour['blue']);
            }
        }
        
        function DrawMap($filename = "")
        {
            // do the basic prep work
            if($this->background != '')
            {
                $image = imagecreatefrompng($this->background);
                if(!$image)
                {
                    print "Failed to open background image\n";
                    die(); 
                }
                else
                {
                    $this->width = imagesx($image);
                    $this->height = imagesy($image);
                }
            }
            else
            {
                $image = imagecreate($this->width, $this->height);
            }
            
            $this->white = imagecolorallocate($image,255,255,255);
	        $this->black = imagecolorallocate($image,0,0,0);
	        $this->grey = imagecolorallocate($image,192,192,192);
            $this->selected = imagecolorallocate($image,255,0,0); // for selections in the editor
            
            $this->AllocateColours($image);
            
            // Now it's time to draw a map
                    
            foreach ($this->links as $link)
            {            
                $link->Draw($image,$this);
            }

            foreach ($this->nodes as $node)
            {
                $node->Draw($image, $this);
            }
          
            $this->DrawLegend($image);
            $this->DrawTimestamp($image, 2, $this->black);
            
            // Ready to output the results...
            
            if($filename == "")
            {  
                imagepng($image);
            }
            elseif ($filename == 'null')
            {
                // do nothing at all - we just wanted the HTML AREAs
            }
            else
            {
                imagepng($image,$filename);
            }
            imagedestroy($image);
        }
    
        function unformat_number($instring)
        {
            $number=0;
            $kilo=$this->kilo;
            
            if(preg_match("/([0-9\.]+)(M|G|K|T)/i",$instring, $matches))
            {
                $number = floatval($matches[1]);
                if($matches[2] == 'K') {  $number = $number * $kilo; }
                if($matches[2] == 'M') {  $number = $number * $kilo * $kilo; }
                if($matches[2] == 'G') {  $number = $number * $kilo * $kilo * $kilo; }              
                if($matches[2] == 'T') {  $number = $number * $kilo * $kilo * $kilo * $kilo; }            
            }
            else
            {
                $number = floatval($instring);
            }
            
            
            return($number);
        }
    
        // These next two are based on perl's Number::Format module
        // by William R. Ward, chopped down to just what I needed
        
        function format_number($number, $precision=2, $trailing_zeroes=0)
        {
            $sign = 1;
            if($number < 0)
            {
                $number = abs($number);
                $sign = -1;
            }
            $number = round($number, $precision);            
            $integer = intval($number);
            
            if (strlen($integer) < strlen($number))
                { $decimal = substr($number, strlen($integer)+1); }
            if(!isset($decimal)) {$decimal="";}
         
            $integer = $sign * $integer;
            if($decimal == "")
            {
                return($integer);
            }
            else
            {
                return($integer.".".$decimal);
            }
        }
        
        function nice_bandwidth($number)
        {
            $suffix = "";
            
            $mega = $this->kilo * $this->kilo;
            $giga = $mega * $this->kilo;
            $tera = $giga * $this->kilo;

            if ($number > $tera)
            {
                $number /= $tera;
                $suffix ="T";
            }            
            elseif ($number > $giga)
            {
                $number /= $giga;
                $suffix ="G";
            }
            elseif ($number > $mega)
            {
                $number /= $mega;
                $suffix = "M";
            }
            elseif ($number > $this->kilo)
            {
                $number /= $this->kilo;
                $suffix = "K";
            }
            $result = $this->format_number($number,1).$suffix;
            return($result);
        }
        
        function MakeHTML()
        {
            $html = "";
            
            $html .= '<HTML><HEAD><META HTTP-EQUIV="REFRESH" CONTENT="300" /><TITLE>'.$this->title.'</TITLE></HEAD><BODY>';
         
           if($this->htmlstyle == "overlib")
           {
              $html .= "<DIV id=\"overDiv\" style=\"position:absolute; visibility:hidden; z-index:1000;\"></DIV>\n";	
              $html .= "<SCRIPT language=\"JavaScript\" src=\"overlib.js\"><!-- overLIB (c) Erik Bosrup --></SCRIPT> \n";

              #   onmouseover="return overlib('<img src=graph.png>',DELAY,250,CAPTION,'$caption');"  onmouseout="return nd();" 
                
                foreach ($this->links as $link)
                {
                    if($link->overliburl != "")
                    {
                        $overlibhtml = "onmouseover=\"return overlib('<img src=".$link->overliburl.">',DELAY,250,CAPTION,'".$link->name."');\"  onmouseout=\"return nd();\"";
                        $this->imap->setProp("extrahtml",$overlibhtml,"LINK:".$link->name);
                    }
                }    
                foreach ($this->nodes as $node)
                {
                    if($node->overliburl != "")
                    {
                        $overlibhtml = "onmouseover=\"return overlib('<img src=".$node->overliburl.">',DELAY,250,CAPTION,'".$node->name."');\"  onmouseout=\"return nd();\"";

                        $this->imap->setProp("extrahtml",$overlibhtml,"NODE:".$node->name);
                    }
                }    
           }

            foreach ($this->links as $link)
            {
                if($link->infourl != "")
                {
                    $this->imap->setProp("href",$link->infourl,"LINK:".$link->name);
                }
            }    
            foreach ($this->nodes as $node)
            {
                if($node->infourl != "")
                {
                    $this->imap->setProp("href",$node->infourl,"NODE:".$node->name);
                }
            }    
           
           // if we're running in "offline" (CLI) mode, then this should be a file, 
           // otherwise it's a link back to us again
           if($this->imagefile == "")
           {
                $html .= sprintf('<IMG SRC="?out=png&config=%s" WIDTH="%s" HEIGHT="%s" BORDER=0 USEMAP="#weathermap_imap" />', 
                $this->configfile,$this->width, $this->height);
            }
            else
            {
                if($this->imageuri != "")
                {
                    $html .= sprintf('<IMG SRC="%s" WIDTH="%s" HEIGHT="%s" BORDER=0 USEMAP="#weathermap_imap" />', 
                        $this->imageuri,$this->width, $this->height);
                }
                else
                {
                    $html .= sprintf('<IMG SRC="%s" WIDTH="%s" HEIGHT="%s" BORDER=0 USEMAP="#weathermap_imap" />', 
                        $this->imagefile,$this->width, $this->height);
                }
            }
            
            $html .= '<MAP NAME="weathermap_imap">';
             
            $html .= $this->imap->subHTML("NODE:");
            $html .= $this->imap->subHTML("LINK:");

            $html .= '</<MAP>';
            $html .= "<hr>Network Map created with <a href=\"http://wotsit.thingy.com/haj/cacti/php-weathermap/\">PHP Network Weathermap</a>";
            $html .= '</BODY></HTML>';
            
            return($html);
        }
        
    };

?>
